--[[
MultiTool Multi Boxing utilities inspired by smurfys MultiBoxer addon

Shouts to http://old.wowace.com/wiki/WelcomeHome_-_Your_first_Ace3_Addon

Extra special shouts to Allara and OrionShock for pointing me in the right direction RE: AceComm-3.0

Author: DigitalSorceress
Date: 2008/11/15
Version: 0.1.04
]]

-- Some initialization of our happy local vars
local myName = "MultiTool"
local myVersion = "0.1.04"

local taxiCommThrottle = false

MultiTool = LibStub("AceAddon-3.0"):NewAddon("MultiTool", "AceConsole-3.0","AceComm-3.0","AceEvent-3.0","AceSerializer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MultiTool")

-- ------------------------------------------------
-- CONFIGURATION STUFF
-- ------------------------------------------------
-- Holds our configuration options
local options = {
  name = "MultiTool",
  handler = MultiTool,
  type = 'group',
  args = {
    enable={
      name=L["ENABLE_NAME"],
      desc=L["ENABLE_DESC"],
      type="toggle",
      get = function(info) return MultiTool.enabled end,
      set = function(info,val) MultiTool.enabled = val end
    },
   debugFlag = {
      type = "toggle",
      name = L["DEBUGGING_NAME"],
      desc = L["DEBUGGING_DESC"],
      get = "getDebugFlag",
      set = "setDebugFlag"
    },
   taxiFlag = {
      type = "toggle",
      name = L["TAXI_NAME"],
      desc = L["TAXI_DESC"],
      get = "getTaxiFlag",
      set = "setTaxiFlag"
    },
   duelFlag = {
      type = "toggle",
      name = L["DUELING_NAME"],
      desc = L["DUELING_DESC"],
      get = "getDuelFlag",
      set = "setDuelFlag"
    },
    myMsg = {
      type = "input",
      name = L["MESSAGE_NAME"],
      desc = L["MESSAGE_DESC"],
      usage = L["MESSAGE_USAGE"],
      get = "getMyMsg",
      set = "setMyMsg"
    },
    showInChat = {
      type = "toggle",
      name = L["SHOW_IN_CHAT"],
      desc = L["SHOW_IN_CHAT_DESC"],
      get = "getShowInChat",
      set = "setShowInChat"
    },
    showOnScreen = {
      type = "toggle",
      name = L["SHOW_ON_SCREEN"],
      desc = L["SHOW_ON_SCREEN_DESC"],
      get = "getShowOnScreen",
      set = "setShowOnScreen"
    }
  }
}

local defaults = {
  profile = {
    enable = true,
    myMsg = L["MESSAGE_DEFAULT"],
    showInChat = false,
    showOnScreen = true,
    debugFlag = false,
    duelFlag = true,
    taxiFlag = true
  }
}


--
-- Field handlers
--
function MultiTool:getMyMsg(info)
  return self.db.profile.myMsg
end

function MultiTool:setMyMsg(info, newValue)
  self.db.profile.myMsg = newValue
end

function MultiTool:getShowInChat(info)
    return self.db.profile.showInChat
end

function MultiTool:setShowInChat(info, value)
    self.db.profile.showInChat = value
end

function MultiTool:getDebugFlag(info)
  return self.db.profile.debugFlag
end
  
function MultiTool:setDebugFlag(info, value)
  self.db.profile.debugFlag = value
end

function MultiTool:getShowOnScreen(info)
    return self.db.profile.showOnScreen
end

function MultiTool:setShowOnScreen(info, value)
    self.db.profile.showOnScreen = value
end

function MultiTool:getDuelFlag(info)
  return self.db.profile.duelFlag
end
  
function MultiTool:setDuelFlag(info, value)
  self.db.profile.duelFlag = value
end

function MultiTool:getTaxiFlag(info)
  return self.db.profile.taxiFlag
end
  
function MultiTool:setTaxiFlag(info, value)
  self.db.profile.taxiFlag = value
end


--
-- Chat Command setup
-- 
function MultiTool:ChatCommand(input)
  if not input or input:trim() == "" then
    -- This would use the Ace config gui
    -- LibStub("AceConfigDialog-3.0"):Open("MultiTool")
    -- This uses the Bliz options pane instead
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(MultiTool, L["SHORT_SLASH_CMD"], L["FULL_SLASH_CMD"], input)
  end
end



--MultiTool.OnCommReceive = {}

-- ------------------------------------------------
-- BASIC REQUIRED FUNCTIONS
-- ------------------------------------------------

-- Called when the addon is loaded
function MultiTool:OnInitialize()
  -- Using the AceDB for storage of our info
  self.db = LibStub("AceDB-3.0"):New("MultiToolDB", defaults, "Default")

  -- main options table stuff
  LibStub("AceConfig-3.0"):RegisterOptionsTable("MultiTool", options)
  -- adding to Blizzard options configuration
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MultiTool", "MultiTool")
  -- registering the slash commands
  self:RegisterChatCommand(L["FULL_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand(L["SHORT_SLASH_CMD"], "ChatCommand")
  self:RegisterChatCommand("help", "helpDialog")
  
  self:RegisterComm("MultiTool","OnCommReceived")


end


-- Called when the addon is enabled
function MultiTool:OnEnable()
  -- just a test register event
  self:RegisterEvent("ZONE_CHANGED", "zoneChanged")
  self:RegisterEvent("DUEL_REQUESTED", "duelRequested")
	
	hooksecurefunc("TakeTaxiNode",takeTaxiNodeHook)

  if (self.db.profile.debugFlag) then
    self:Print(myName.." v"..myVersion.." ENABLED!")
  end
end


-- Called when the addon is disabled
function MultiTool:OnDisable()
  --
end


-- ------------------------------------------------
-- EVENT HANDLERS
-- ------------------------------------------------

function MultiTool:zoneChanged()
  if GetBindLocation() == GetSubZoneText() then
    -- CHAT
    if (self.db.profile.showInChat) then
      self:Print(self.db.profile.message)
    end
    
    -- IN-YER-FACE
    if (self.db.profile.showOnScreen) then
      UIErrorsFrame:AddMessage(self.db.profile.message, 1.0, 1.0, 1.0, 5.0)
    end
  end
end


--
-- If profile is set for denying duels, then autocancels all incoming duels
--
function MultiTool:duelRequested(challenger)
	if (self.db.profile.duelFlag) then
    if (self.debugFlag) then
      self:Print("Duel denied from challenger:"..challenger)
    end
    CancelDuel();
  	StaticPopup_Hide("DUEL_REQUESTED")
	end
end

-- ------------------------------------------------
-- HOOKED FUNCTIONS
-- ------------------------------------------------
function takeTaxiNodeHook(id, noComm) 
  local self = MultiTool
    
  -- Only use the additional functionality IF taxiFlag is true
  if (self.db.profile.taxiFlag) then
    local prefix = "MultiTool"
    local msg = {
      op = "TAKETAXI",
      node = id,
      nodeName = TaxiNodeName(id)
    }
    local text = self:Serialize(msg)
    
    local distribution = "RAID"
    local sender = player
    local prio = "ALERT"
    
    --debug
    if (self.db.profile.debugFlag) then
     	self:Print("SentBroadcast "..msg["node"].." : "..msg["nodeName"])
    	self:Print("msg object: "..text)
    end
    -- send the command to take my taxi
    -- New throttling - DO NOT send msg unless noComm is nil
    if (noComm) then
      if (self.db.profile.debugFlag) then
        self:Print("noComm flag IS set, NOT sending command")
      end
      return
    else
      if (self.db.profile.debugFlag) then
        self:Print("noComm flag NOT set, sending command")
      end
      self:SendCommMessage(prefix, text, distribution, sender, prio)
    end
  else
    if (self.db.profile.debugFlag) then
      self:Print("taxiFlag is false... not sending comms")
    end
  end
end -- end function takeTaxiNodeHook

-- ------------------------------------------------
-- COMM-BASED FUNCTIONS
-- ------------------------------------------------

function MultiTool:OnCommReceived(prefix, message, distribution, sender)
  local success, o = self:Deserialize(message)
  if (self.db.profile.debugFlag) then
  	self:Print("comm received")
  	self:Print("  prefix: "..prefix)
  	self:Print("  message: "..message)
  	self:Print("  distribution: "..distribution)
  	self:Print("  sender: "..sender)
  	self:Print("  player: "..UnitName("player"))
  	self:Print("  Deserialize success: "..tostring(success))
  	self:Print("  Deserialize o: "..tostring(o))
  	--self:Print("  priority: "..priority)
  end -- end if debugFlag

  -- don't do anyting if we could not deserialize
  if (success == false) then
    if (self.db.profile.debugFlag) then
      self:Print("Failed to deserialize the message - OnCommReceived can not process any further")
    end -- if debugFlag
    return -- Failure
  else
    if (o["op"] == "TAKETAXI") then

      -- Only continue IF the taxiFlag is true
      if (self.db.profile.taxiFlag) then
  
      -- Ok, This is a BIG  bunch of stuff needed to stop a message storm
      -- the idea here is that we do NOT want to continue to try to take taxi
      -- (and thus re-calling the message send)
      -- IF we are the message originator or if the node we picked is invalid
      -- or if we're not currently at a taxi node or if we're currently ON a 
      -- taxi. 

        -- removed   taxiCommthrottle == true condition 
        if (  UnitOnTaxi("player") 
              or o["nodeName"] == "INVALID" 
              or o["nodeName"] ~= TaxiNodeName(o["node"])
              or sender == UnitName("player")
        ) 
        then
          -- safe to turn off the taxi comm throttle cuz we arent sending
          if (self.db.profile.debugFlag) then
            self:Print("TAKETAXI op NOT engaged")
            self:Print("  Am I on taxi? "..tostring(UnitOnTaxi("player")))
            self:Print("  actual node requested: "..tostring(o["node"]).." ("..o["nodeName"]..")")
            self:Print("  effective node requested: "..tostring(o["node"]).." ("..TaxiNodeName(o["node"])..")")
          end
          -- I think I can remove this 
          -- taxiCommThrottle = false
          return -- DON'T DO ANYTHING: this stops a message storm
        else
          if (self.db.profile.debugFlag) then
            self:Print("TAKETAXI op engaged")
            self:Print("  setting taxiCommThrottle to true")
          end
          -- I think I can 
          -- taxiCommThrottle = true
          -- by passing the extra arg "noComm" to the takeTaxiNode, I think I solve the msg storms
          TakeTaxiNode(o["node"], "noComm")
        end -- if taxiCommTrhottle.. else
      else
        self:Print("taxiFlag is false... ignoring TAKETAXI request")
      end -- end if taxiFlag
    end -- end if op==TAKETAXI
  end -- if success==false
end -- function 

-- ------------------------------------------------
-- COMM-BASED FUNCTIONS
-- ------------------------------------------------
function MultiTool:helpCommand()
  self:Print("MultTool Help")
  self:Print("")
  self:Print("COMMANDS:")
  self:Print("  /MultiTool help - Show this help menu")
  self:Print("")
end
